#include <def21369.h>

#define N 100
.section/dm seg_sdram;
.var destination[N];

.section/dm seg_dmda;
	.var SourceData[N];
	.var DestData[N];
	.var int_count = 0;
		
.section/pm seg_rth;
 	nop;nop;nop;nop;nop;jump start;

// External Port DMA Channel 0 interrupt service routine
.section/pm seg_epdma0;
	jump ep_isr;
	rti;
	rti;
	rti;

.section/pm seg_pmco;	
ep_isr:	
	r3 = dm(int_count);
	r3 = r3 + 1;
	dm(int_count) = r3;
	rti;
		
.section/pm seg_pmco;
start:
	call _initPLL;
	call _initSDRAM;
	
	r0 = 0x0;
	i1 = SourceData;
	m1 = 1;
	l1 = 0;
	lcntr = N, do SourceInit until lce;
	dm(i1,m1) = r0;
SourceInit:	r0 = r0 + 1;

	// Enabling DMA interrupt
	bit set mode1 IRPTEN;
	bit set lirptl P9IMSK; //Enable programmable interrupt "9" - default: EPDMA 0 Interrupt
	nop;nop;nop;nop;
	
	// dummy READ from SDRAM to start the power up sequence
	nop; nop; nop;
	r13 = dm(0x08000000); //This initiates the power on sequence of SDRAM
	
	/*Set Flasg0 : Tigger LA to capture Power Sequence*/
	//bit set FLAGS FLG0; //Drive HI
	/*---------*/
	
	nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;
	nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;

	//This is DMA mode Write to SDRAM and Read from SDRAM
	CALL IntToExtDMA;
	idle; //Wait for DMA completion Interrupt
	
	CALL ExtToIntDMA;
	idle; //Wait for DMA completion Interrupt
	
loophere:	
	jump loophere;

//^^^^^^^^^^^^^^^^^^^Int To Ext EP DMA Configuration subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
IntToExtDMA:
	r0 = 0;
	dm(DMAC0) = r0; //Clear DMAC0 - EP DMA Channel0 control register
	
	r0 = SourceData;
	dm(IIEP0) = r0; //Load Internal Memory Index 
	
	r0 = 1;
	dm(IMEP0) = r0; //Load Internal Memory Modifier
	
	r0 = N;
	dm(CEP0) = r0;  //Load Count : Number of words transfer
	
	r0 = destination;
	dm(EIEP0) = r0; //Load External Memory Index
	
	r0 = 1;
	dm(EMEP0) = r0; //Load External Modifier
	/* Flag0 driven HI to trigger LA - Only for test purpose */
	//bit set FLAGS FLG0; //Drive HI
	/*-------------------------------------------------------*/
	
	// Now setup the DMA control registers
	r0 = DEN | DFLSH | TRAN; //DMADR / TRAN is "1" : Write to External Memory
	dm(DMAC0) = r0;
	
	RTS;
	

//^^^^^^^^^^^^^^^^^^^Ext To Int EP DMA Configuration subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
ExtToIntDMA:

	r0 = 0;
	dm(DMAC0) = r0; //Clear DMAC0 - EP DMA Channel0 control register
	
	r0 = DestData;
	dm(IIEP0) = r0; //Load Internal Memory Index 
	
	r0 = 1;
	dm(IMEP0) = r0; //Load Internal Memory Modifier
	
	r0 = N;
	dm(CEP0) = r0;  //Load Count : Number of words transfer
	
	r0 = destination;
	dm(EIEP0) = r0; //Load External Memory Index
	
	r0 = 1;
	dm(EMEP0) = r0; //Load External Modifier
	
	
	// Now setup the DMA control registers
	r0 = DEN | DFLSH; //DMADR is "0" : Write to Internal Memory
	dm(DMAC0) = r0;
	
	RTS;

	
_initPLL:

// CLKIN= 24.576 MHz, Multiplier= 27, Divisor= 2, CCLK_SDCLK_RATIO 2.5.
// Core clock = (24.576MHz * 27) /2 = 331.776 MHz
// Set the Core clock (CCLK) to SDRAM clock (SDCLK) ratio to 2.5
    ustat3 = PLLM27|PLLD2|DIVEN|SDCKR2_5;

changePLL:

    dm(PMCTL) = ustat3;
    bit set ustat3 PLLBP;
    bit clr ustat3 DIVEN;
    dm(PMCTL) = ustat3;

     // Wait for at least 4096 cycles for the pll to lock
    lcntr = 5000, do loopend2 until lce;
loopend2:     nop;

    ustat3 = dm(PMCTL);
    bit clr ustat3 PLLBP;
    dm(PMCTL) = ustat3;

_initPLL.end: rts;


_initSDRAM:

    // Programming SDRAM control registers.
    // RDIV = ((f SDCLK X t REF )/NRA) - (tRAS + tRP )

    // CCLK_SDCLK_RATIO==2.5
    ustat4 = 0x814; // (133*(10^6)*64*(10^-3)/4096) - (7+3) = 2068
     

    //===================================================================
    //
    // Configure SDRAM Control Register (SDCTL) for the Micron MT48LC4M32
    //
    //  SDCL3  : SDRAM CAS Latency= 3 cycles
    //  DSDCLK1: Disable SDRAM Clock 1
    //  SDPSS  : Start SDRAM Power up Sequence
    //  SDCAW8 : SDRAM Bank Column Address Width= 8 bits
    //  SDRAW12: SDRAM Row Address Width= 12 bits
    //  SDTRAS7: SDRAM tRAS Specification. Active Command delay = 7 cycles
    //  SDTRP3 : SDRAM tRP Specification. Precharge delay = 3 cycles.
    //  SDTWR2 : SDRAM tWR Specification. tWR = 2 cycles.
    //  SDTRCD3: SDRAM tRCD Specification. tRCD = 3 cycles.
    //
    //--------------------------------------------------------------------

    ustat3 = SDCL3|DSDCLK1|SDPSS|SDCAW8|SDRAW12|SDTRAS7|SDTRP3|SDTWR2|SDTRCD3;

    dm(SDCTL) = ustat3;

    // Change this value to optimize the performance for quazi-sequential accesses (step > 1)
    #define SDMODIFY 1                    // Setting the Modify to 1
    bit set ustat4 (SDMODIFY<<17)|SDROPT; // Enabling SDRAM read optimization

    dm(SDRRC) = ustat4;

    // Note that MS2 & MS3 pin multiplexed with flag2 & flag3.
    // MSEN bit must be enabled to access SDRAM, but LED8 cannot be driven with sdram
    ustat3=dm(SYSCTL);
    bit set ustat3 MSEN;    //This setting allows SDRAM access
    //bit clr ustat3 MSEN;  //This setting allows Flag3 to be used
    dm(SYSCTL)=ustat3;

    // Mapping Bank 2 to SDRAM
    // Make sure that jumper is set appropriately so that MS2 is connected to
    // chip select of 16-bit SDRAM device
    ustat3 = dm(EPCTL);
    bit set ustat3 B2SD;
    bit clr ustat3 B0SD|B1SD|B3SD;
    dm(EPCTL) = ustat3;

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL0) Bank 0 for the ISSI IS61LV5128
    //
    //  WS2 : Wait States= 2 cycles
    //  HC1  : Bus Hold Cycle (at end of write access)= 1 cycle.
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------

    //SRAM Settings
    ustat4 = WS2|HC1|AMIEN|BW8;
    dm(AMICTL0)= ustat4;

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL) Bank 1 for the AMD AM29LV08
    //
    //  WS23 : Wait States= 23 cycles
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------

    //Flash Settings
    ustat4 = WS23|AMIEN|BW8;
    dm(AMICTL1)= ustat4;


_initSDRAM.end: rts;

